module AkidoLib
  module ReferenceInstanceExtensions
    def read
      model, id = reference.match(/(\D+)\/(\d+)/).captures
      cached_models = Redis::HashKey.new(model)
      if matching_cached_model = cached_models[id]
        return "FHIR::#{model}".constantize.new(JSON.parse(matching_cached_model))
      else
        model = super
        cached_models[model.id] = model.to_json
        return model
      end
    end
  end

  module ModelClassExtensions
    def cache!
      model = self.to_s.demodulize
      cache = Redis::HashKey.new(model)

      records = self.search.to_a
      records.each do |record|
        cache[record.id] = record.to_json

      end
    end

    def cached
      model = self.class.to_s.demodulize

      Redis::HashKey.new(model).values.map do |record|
        self.new(JSON.parse(record))
      end
    end
  end
end

class FHIR::Reference
  prepend AkidoLib::ReferenceInstanceExtensions
end

FHIR::Model.singleton_class.prepend AkidoLib::ModelClassExtensions
