module AkidoLib
  class Query
    FILTERS = [
      :content, 
      :id, 
      :last_updated, 
      :profile, 
      :query, 
      :security, 
      :tag,
      :text,
      :filter
    ]

    def initialize(resource, base_url)
      @base_url = if base_url
                    base_url
                  elsif @@base_url
                    @@base_url
                  else
                    AkidoLib::Model
                      .client
                      .full_resource_url(:resource => @model)
                  end

      @resource = resource
      @query = []
    end

    def where(filters)
      filters.each do |filter, value|
        if FILTERS.include?(filter.to_s.underscore.downcase.to_sym)
          @query.push({ "_#{filter.to_s.camelize(:lower)}".to_sym => value })
        end
      end

      return self
    end

    def include(relation, options)
      key = if options[:rev]
              :_revinclude
            else
              :_include
            end

      @query.push({key => relation})
    end

    def last_updated(date)
      @query.push({:_lastUpdated => date.xmlschema})
    end

    def custom(key, value)
      @query.push({key => value})
    end

    def unset(key)
      key = "_#{key.to_s.camelize(:lower)}".to_sym

      @query.reject! do |param| 
        param.keys.include?(key)
      end
    end

    def url
      "#{@base_url}/#{resource_name}?#{query_string}"
    end

    def fetch!
    end
    
    def self.set_base_url!(base_url)
      @@base_url = base_url
    end

    private
    def resource_name
      @resource.to_s.demodulize
    end

    def query_string
      @query.map(&:to_query).join("&")
    end
  end
end
