module RestClient
  @@config = {}

  def self.get_config
    @@config
  end

  def self.set_config(**config)
    config.each do |key, value|
      @@config[key] = value unless key == :url || key == :method
    end
  end

  def self.clear_config
    @@config = {}
  end

  def self.get(url, headers={}, &block)
    Request.execute(:method => :get, :url => url, :headers => headers, **@@config, &block)
  end

  def self.post(url, payload, headers={}, &block)
    Request.execute(:method => :post, :url => url, :payload => payload, :headers => headers, **@@config, &block)
  end

  def self.patch(url, payload, headers={}, &block)
    Request.execute(:method => :patch, :url => url, :payload => payload, :headers => headers, **@@config, &block)
  end

  def self.put(url, payload, headers={}, &block)
    Request.execute(:method => :put, :url => url, :payload => payload, :headers => headers, **@@config, &block)
  end

  def self.delete(url, headers={}, &block)
    Request.execute(:method => :delete, :url => url, :headers => headers, **@@config, &block)
  end

  def self.head(url, headers={}, &block)
    Request.execute(:method => :head, :url => url, :headers => headers, **@@config, &block)
  end

  def self.options(url, headers={}, &block)
    Request.execute(:method => :options, :url => url, :headers => headers, **@@config, &block)
  end
end
